#ifndef _KILIB_TEXTFILE_H_
#define _KILIB_TEXTFILE_H_
#include "types.h"
#include "ktlaptr.h"
#include "memory.h"
#include "file.h"
#ifndef __ccdoc__
namespace ki {
#endif



//=========================================================================
//@{ @pkg ki.StdLib //@}
//@{
//	p\R[hZbg
//
//	AŃXgAbvꂽ̂̂AWindowsɂ
//	T|[gCXg[Ă̂ۂɂ͑Ή\B
//	l-100菬R[h́ÂɂR[hy[W̌
//	T|[g𗘗pĕϊs߁AɈˑB
//@}
//=========================================================================

enum charset {
	AutoDetect = 0,    // 
	                   // SJIS/EucJP/IsoJP/IsoKR/IsoCN
					   // UTF5/UTF8/UTF8N/UTF16b/UTF16l/UTF32b/UTF32l
					   // 𔻒肷B͒mȂB(^^;

	Western    = -1,   //       (Windows1252 >> ISO-8859-1)
	Turkish    = 1254, // gR  (Windows1254 >> ISO-8859-9)
	Hebrew     = 1255, // wuC(Windows1255 >> ISO-8859-8)
	Arabic     = 1256, // ArA(Windows1256 ` ISO-8859-6)
	Baltic     = 1257, // og  (Windows1257 >> ISO-8859-13)
	Vietnamese = 1258, // xgi(Windows1258 != VISCII)
	Central    = 1250, // ְۯ(Windows1250 ` ISO-8859-2)
	Greek      = 1253, // MV(Windows1253 ` ISO-8859-7)
	Thai       = 874,  // ^C

	Cyrillic   = 1251, // L(Windows1251 != ISO-8859-5)
	Koi8R      = 20866,// L(KOI8-R)
	Koi8U      = 21866,// L(KOI8-U ENCin)

	UHC        = 949,  // ؍P (Unified Hangle Code >> EUC-KR)
	IsoKR      = -950, // ؍Q (ISO-2022-KR)
	Johab      = 1361, // ؍R (Johab)

	GBK        = 936,  // P (ȑ̎ GBK >> EUC-CN)
	IsoCN      = -936, // Q (ȑ̎ ISO-2022-CN)
	HZ         = -937, // R (ȑ̎ HZ-GB2312)
	Big5       = 950,  // S (ɑ̎ Big5)

	SJIS       = 932,  // {P (Shift_JIS)
	EucJP      = -932, // {Q ({EUC)
	IsoJP      = -933, // {R (ISO-2022-JP)

	UTF5       = -2,   // Unicode  (UTF-5)   : BOM
	UTF7       = 65000,// Unicode  (UTF-7)   : BOM
	UTF8       =-65001,// Unicode  (UTF-8)   : BOML
	UTF8N      = 65001,// Unicode  (UTF-8N)  : BOM
	UTF16b     = -3,   // Unicode  (UTF-16)  : BOML BE
	UTF16l     = -4,   // Unicode  (UTF-16)  : BOML LE
	UTF16BE    = -5,   // Unicode  (UTF-16BE): BOM
	UTF16LE    = -6,   // Unicode  (UTF-16LE): BOM
	UTF32b     = -7,   // Unicode  (UTF-32)  : BOML BE
	UTF32l     = -8,   // Unicode  (UTF-32)  : BOML LE
	UTF32BE    = -9,   // Unicode  (UTF-32BE): BOM
	UTF32LE    = -10,  // Unicode  (UTF-32LE): BOM

	DOSUS      = 437   // DOSLatinUS (CP437)
};

//=========================================================================
//@{
//	sR[h
//@}
//=========================================================================

enum lbcode {
	CR   = 0,
	LF   = 1,
	CRLF = 2
};

struct TextFileRPimpl;
struct TextFileWPimpl;



//=========================================================================
//@{
//	eLXgt@CǍ
//
//	t@Cw肳ꂽR[hŉ߂AUnicodeƂ
//	sɕԂBR[hsR[h̎\B
//@}
//=========================================================================

class TextFileR : public Object
{
public:

	//@{ RXgN^iR[hwj//@}
	TextFileR( int charset=AutoDetect );

	//@{ fXgN^ //@}
	~TextFileR();

	//@{ J //@}
	bool Open( const TCHAR* fname );

	//@{  //@}
	void Close();

	//@{
	//	ǂݍ (ǂ񂾒Ԃ)
	//
	//	ȂƂ20炢̃TCYmۂobt@w肵ĂB
	//@}
	size_t ReadLine( unicode* buf, ulong siz );

public:

	//@{ ǂłt@C̃R[hy[W //@}
	int codepage() const;

	//@{ sR[h (0:CR, 1:LF, 2:CRLF) //@}
	int linebreak() const;

	//@{ ǂݍݏ (0:EOF, 1:EOL, 2:EOB) //@}
	int state() const;

	//@{ t@CTCY //@}
	ulong size() const;

private:

	dptr<TextFileRPimpl> impl_;
	FileR                fp_;
	int                  cs_;
	int                  lb_;

private:

	int AutoDetection( int cs, const uchar* ptr, ulong siz );

private:

	NOCOPY(TextFileR);
};



//-------------------------------------------------------------------------

inline int TextFileR::codepage() const
	{ return cs_; }

inline int TextFileR::linebreak() const
	{ return lb_; }

inline ulong TextFileR::size() const
	{ return fp_.size(); }



//=========================================================================
//@{
//	eLXgt@C
//
//	Unicode󂯎Aw肳ꂽR[hɕϊȂo͂B
//@}
//=========================================================================

class TextFileW : public Object
{
public:

	//@{ RXgN^i,sR[hwj//@}
	TextFileW( int charset, int linebreak );
	~TextFileW();

	//@{ J //@}
	bool Open( const TCHAR* fname );

	//@{  //@}
	void Close();

	//@{ so //@}
	void WriteLine( const unicode* buf, ulong siz, bool lastline );

private:

	dptr<TextFileWPimpl> impl_;
	FileW                fp_;
	const int            cs_;
	const int            lb_;

private:

	NOCOPY(TextFileW);
};



//=========================================================================

}      // namespace ki
#endif // _KILIB_TEXTFILE_H_
